/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "TinyCadView.h"
#include "diag.h"
#include "colour.h"

#define theWnd ((CMainWindow)dc.theWindow)


// This defines the block edit functions

// The rectangle area slightly re-written
#define theArea CRect(a.x,a.y,b.x,b.y)




////// The block move object //////

void CDrawBlockMove::Paint(Context &dc,paint_options options)
{

  dc.SelectBrush();
  dc.SelectPen(PS_DOT,1,cBLOCK);
	dc.SetROP2(R2_COPYPEN);

  dc.Rectangle(theArea);
}


CDrawBlockMove::CDrawBlockMove(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
  segment=1;
  placed=0;
  a=b=CPoint(0,0);
}

void CDrawBlockMove::EndEdit()
{

  // Cancel the place operation
  if (placed)
	Move(OldPos);

  if (!segment | placed) {
	m_pDesign->UnSelect();
	Display();
  }

  m_pDesign->Invalidate();
}

void CDrawBlockMove::Place(CPoint p, CPoint)
{
  if (placed) {
	Display();
	m_pDesign->SelectUndoMove(CPoint(OldPos.x-b.x,OldPos.y-b.y));
	m_pDesign->UnSelect();
	segment=1;
 	placed=0;
	a = b;
	m_pDesign->SetModifiedFlag( TRUE );
	m_pDesign->Invalidate();
  } else if (segment) {
	a=p;
	b=p;
	segment=0;
	placed=0;
	Display();
  } else {
	Display();
	b=p;
	placed=1;
	segment=0;
	// Now select all the objects in this rectangle
	m_pDesign->Select(a,b);
	OldPos = b;
	Display();
  }
}


void CDrawBlockMove::Move(CPoint p)
{
  if (placed) {
  	CPoint r;

  	r.x=p.x-b.x;
  	r.y=p.y-b.y;

	if (r.x!=0 || r.y!=0) {
		Display();
		m_pDesign->SelectMove(r);
		a.x += r.x;
		a.y += r.y;
		b=p;
		Display();
	}
  } else if (!segment) {
	Display();
	b=p;
	Display();
  }

}



BOOL CDrawBlockMove::End()
{
  BOOL r = (m_pDesign->GetSelect()!=NULL);

  // Cancel the place operation
  if (placed)
	Move(OldPos);

  // If necessary remove the line from the screen
  if (!segment | placed) {
	Display();
	m_pDesign->UnSelect();
	segment=1;
 	placed=0;
	a = b;
  }

  return r;
}




////// The rotate move object //////

void CDrawBlockRotate::Paint(Context &dc,paint_options options)
{

  dc.SelectBrush();
  dc.SelectPen(PS_DOT,1,cBLOCK);
	dc.SetROP2(R2_COPYPEN);

  dc.Rectangle(theArea);
}


CDrawBlockRotate::CDrawBlockRotate(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
  segment=1;
  placed=0;
  a=b=CPoint(0,0);
}


void CDrawBlockRotate::BeginEdit()
{
}

void CDrawBlockRotate::EndEdit()
{
  if (placed)
	g_EditToolBar.m_RotateBox.Close();

  if (!segment | placed) {
	m_pDesign->UnSelect();
	Display();
	segment = 1;
	placed = 0;
  }
}

void CDrawBlockRotate::Place(CPoint p, CPoint)
{
  if (placed) {
	Display();
	m_pDesign->UnSelect();
	g_EditToolBar.m_RotateBox.Close();
	m_pDesign->SetModifiedFlag( TRUE );
	segment=1;
 	placed=0;
	a = b;
  } else if (segment) {
	a=p;
	b=p;
	segment=0;
	placed=0;
	Display();
  } else {
	Display();
	b=p;
	placed=1;
	segment=0;
	// Now select all the objects in this rectangle
	m_pDesign->Select(a,b);
	g_EditToolBar.m_RotateBox.Open(m_pDesign,this);

	// Build the Undo list
	m_pDesign->SelectUndoMove(CPoint(0,0));

	Display();
  }
}


void CDrawBlockRotate::Move(CPoint p)
{
  if (!placed && !segment) {
	Display();
	b=p;
	Display();
  }
}


void CDrawBlockRotate::ChangeDir(int NewDir)
{
  CDrawingObject *pointer = m_pDesign->GetHead();

  if (!m_pDesign->IsSelected())
	return;

  // Find the centre of rotation
  CPoint centre = m_pDesign->m_snap.Snap(CPoint((b.x-a.x)/2+a.x,(b.y-a.y)/2+a.y));

  Display();

  while (pointer!=NULL) {
	pointer->Rotate(centre,NewDir);
	if (pointer==m_pDesign->GetSelect())
		break;
	pointer= pointer->next;
  }


  Rotate(centre,NewDir);

  Display();

  m_pDesign->Invalidate();
}


BOOL CDrawBlockRotate::End()
{
  BOOL r = (m_pDesign->GetSelect()!=NULL);

  // Cancel the rotate operation
  if (placed) {
	g_EditToolBar.m_RotateBox.Close();
	a = b;
  }

  // If necessary remove the line from the screen
  if (!segment | placed) {
	Display();
	m_pDesign->UnSelect();
	segment=1;
 	placed=0;
  }

  return r;
}




////// The block duplicate object //////

void CDrawBlockDup::Paint(Context &dc,paint_options options)
{

  dc.SelectBrush();
  dc.SelectPen(PS_DOT,1,cBLOCK);
	dc.SetROP2(R2_COPYPEN);

  dc.Rectangle(theArea);
}


CDrawBlockDup::CDrawBlockDup(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
  segment=1;
  placed=0;
  a=b=CPoint(0,0);
}

void CDrawBlockDup::EndEdit()
{
  if (!segment | placed) {
	m_pDesign->UnSelect();
	Display();
  }
}

void CDrawBlockDup::Place(CPoint p, CPoint)
{
  if (placed) {
	Display();
	CDrawingObject *ObjPtr = m_pDesign->GetHead();
	CDrawingObject *first = NULL;
	// Make another duplicate, and place the first!
	if (m_pDesign->IsSelected())
		while (ObjPtr != NULL) {
			ObjPtr->Store();
			ObjPtr->Display();
			if (first == NULL)
				first = m_pDesign->GetHead();
			if (ObjPtr==m_pDesign->GetSelect())
				break;
			ObjPtr = ObjPtr->next;
		}
	if (first != NULL)
		m_pDesign->SetSelect(first);
	a = b;
	m_pDesign->Invalidate();
	m_pDesign->SetModifiedFlag( TRUE );
	Display();
  } else if (segment) {
	a=p;
	b=p;
	segment=0;
	placed=0;
	Display();
  } else {
	Display();
	b=p;
	placed=1;
	segment=0;
	// Now select all the objects in this rectangle
	m_pDesign->Select(a,b);
	CDrawingObject *ObjPtr = m_pDesign->GetHead();
	CDrawingObject *first = NULL;
	if (m_pDesign->IsSelected())
		while (ObjPtr != NULL) {
			ObjPtr->Store();
			ObjPtr->Display();
			if (first == NULL)
				first = m_pDesign->GetHead();
			if (ObjPtr==m_pDesign->GetSelect())
				break;
			ObjPtr = ObjPtr->next;
		}
	if (first != NULL)
		m_pDesign->SetSelect(first);
	Display();
  }
}


void CDrawBlockDup::Move(CPoint p)
{
  if (placed) {
  	CPoint r;

  	r.x=p.x-b.x;
  	r.y=p.y-b.y;

	if (r.x!=0 || r.y!=0) {
		Display();
		m_pDesign->SelectMove(r);
		a.x += r.x;
		a.y += r.y;
		b=p;
		Display();
	}
  } else if (!segment) {
	Display();
	b=p;
	Display();
  }
}



BOOL CDrawBlockDup::End()
{
  BOOL r = (m_pDesign->GetSelect()!=NULL);

  // If necessary remove the line from the screen
  if (!segment | placed) {
	Display();
	if (placed)
		m_pDesign->SelectDelete();
	segment=1;
 	placed=0;
	a = b;
	m_pDesign->Invalidate();
  }

  return r;
}





////// The block drag object //////

void CDrawBlockDrag::Paint(Context &dc,paint_options options)
{

  dc.SelectBrush();
  dc.SelectPen(PS_DOT,1,cBLOCK);
	dc.SetROP2(R2_COPYPEN);

  dc.Rectangle(theArea);
}


CDrawBlockDrag::CDrawBlockDrag(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
  head=NULL;
  segment=1;
  placed=0;
  a=b=CPoint(0,0);
}

void CDrawBlockDrag::EndEdit()
{
  if (!segment | placed) {
	// Undo the drag
	Move(OldPos);

	m_pDesign->UnSelect();
	if (head!=NULL) {
		delete head;
		head=NULL;
	}
	Display();
  }
  m_pDesign->Invalidate();
}

void CDrawBlockDrag::Place(CPoint p, CPoint)
{
  if (placed) {
	Display();
	m_pDesign->UnSelect();
	if (head!=NULL) {
		delete head;
		head=NULL;
	}
	segment=1;
 	placed=0;
	a = b;
	m_pDesign->SetModifiedFlag( TRUE );
  } else if (segment) {
	a=p;
	b=p;
	segment=0;
	placed=0;
	Display();
  } else {
	Display();
	b=p;
	OldPos = p;
	// Now select all the objects in this rectangle
	m_pDesign->Select(a,b);
	if (m_pDesign->GetSelect()==NULL) {
		Display();
		End();
		return;
	}

	placed=1;
	segment=0;

	// Build the Undo List
	m_pDesign->SelectUndoMove(CPoint(0,0));

	// Search for Methods which might be attached to wires via pins
	IntList *lst,*last = NULL;
	head = NULL;

	CDrawingObject *obj=m_pDesign->GetHead();
	while (obj!=NULL) {
		// Is this a symbol with wires attached to it's pins which need to be dragged?
		if (obj->GetType()==xMethodEx2) {
			#define thePin ((CDrawPin*)MethodPtr)
			#define theMethod ((CDrawMethod*)obj)

			CPoint tr;
			CDrawingObject *MethodPtr = theMethod->ExtractSymbol(tr);
			m_pDesign->SetPart(theMethod->GetSubPart());
			
			// Search the symbol for pins
			while (MethodPtr!=NULL) {
				// If it is a pin then search for wires attached
				if (MethodPtr->GetType()==xPinEx && !(thePin->IsInvisible())) {
					CPoint hs = thePin->GetPosition(theMethod);
					// If this pin is inside the selected area then continue
					if (   hs.x>=min(a.x,b.x) && hs.x<=max(a.x,b.x)
					    && hs.y>=min(a.y,b.y) && hs.y<=max(a.y,b.y) ) {
						MethodPtr = MethodPtr->next;
						continue;
					}

					// Find any wires attached to this pin and select them
					CDrawingObject *pointer = m_pDesign->GetSelect(), *lastpointer = NULL, *next;

					// Only search unselected objects
					lastpointer = pointer;
					pointer = pointer->next;
					while (pointer!=NULL) {
						next = pointer->next;
						if (pointer->GetType()==xWire && (pointer->a == hs || pointer->b==hs)) {
							// Found one! Select it...
							lastpointer->next = next;
							pointer->next = m_pDesign->GetHead();
							m_pDesign->SetHead(pointer);

							// Now display it highlighted
							pointer->Display();

							// Build the undo list
							CDrawingObject *OldSelect = m_pDesign->GetSelect();
							m_pDesign->SetSelect(pointer);
							m_pDesign->SelectDelete();
							pointer->Store();
							pointer = m_pDesign->GetHead();
							m_pDesign->SetSelect(OldSelect);
						
							// Now add this to the list of unmovable points
							lst = new IntList(pointer,pointer->a == hs,pointer->b == hs);
							lst->next = head;
							head = lst;
							if (last == NULL)
								last = lst;
							;
						} else
							lastpointer=pointer;
						pointer=next;
					}
				}
				MethodPtr=MethodPtr->next;
			}
		}

		if (obj==m_pDesign->GetSelect())
			break;
		obj=obj->next;
	}

	// Search for wires which might need one point fixed in place
	BOOL MoveA,MoveB;
	obj = m_pDesign->GetHead();
	while (obj!=NULL) {
		MoveA=FALSE;
		MoveB=FALSE;
		// Is this a wire which needs to be draged?
		if (obj->GetType()==xWire || obj->GetType()==xBus) {
			if (   obj->a.x>=min(a.x,b.x) && obj->a.x<=max(a.x,b.x)
			    && obj->a.y>=min(a.y,b.y) && obj->a.y<=max(a.y,b.y) )
				MoveA = TRUE;

			if (   obj->b.x>=min(a.x,b.x) && obj->b.x<=max(a.x,b.x)
			    && obj->b.y>=min(a.y,b.y) && obj->b.y<=max(a.y,b.y) )
				MoveB = TRUE;

			// Only include in the list if only one out of the two points
			// needs to be moved
			if (MoveA != MoveB) {
				lst=new IntList(obj,MoveA,MoveB);
				if (last==NULL)
					head=lst;
				else
					last->next=lst;
				last=lst;
			}
		}
		if (obj==m_pDesign->GetSelect())
			break;
		obj=obj->next;
	}

	// Now display the drag box
	Display();
	}
}


void CDrawBlockDrag::Move(CPoint p)
{
  if (placed) {
  	CPoint r;

  	r.x=p.x-b.x;
  	r.y=p.y-b.y;

	if (r.x!=0 || r.y!=0) {
		Display();
		CDrawingObject *obj=m_pDesign->GetHead();
		IntList *lst=head;

		// With drag only move the points of a wire inside the update region

		// Remove all the objects from the screen
		while (obj!=NULL) {
			// Change the object to XOR
			obj->Display();
			if (lst==NULL || obj!=lst->obj) {
				// Move the object
				obj->a.x += r.x;
				obj->a.y += r.y;
				obj->b.x += r.x;
				obj->b.y += r.y;
			} else {
				if (lst->MoveA) {
					obj->a.x += r.x;
					obj->a.y += r.y;
				}
				if (lst->MoveB) {
					obj->b.x += r.x;
					obj->b.y += r.y;
				}
				lst=lst->next;
			}
		  	if (obj==m_pDesign->GetSelect())
				break;
			obj=obj->next;
		}
  
  		obj=m_pDesign->GetHead();
		while (obj!=NULL) {
			// Change the object to XOR
			obj->Display();
			if (obj==m_pDesign->GetSelect())
				break;
			obj=obj->next;
		} 

		a.x += r.x;
		a.y += r.y;
		b=p;
		Display();
	}
  } else if (!segment) {
	Display();
	b=p;
	Display();
  }

}



BOOL CDrawBlockDrag::End()
{
  BOOL r = (m_pDesign->GetSelect()!=NULL);

  // If necessary remove the line from the screen
  if (!segment | placed) {
	// Undo the drag
	Move(OldPos);
	Display();
	m_pDesign->UnSelect();
	if (head!=NULL) {
		delete head;
		head=NULL;
	}
	segment=1;
 	placed=0;
	a = b;
  }

  return r;
}




////// The block export object //////

void CDrawBlockExport::Paint(Context &dc,paint_options options)
{
  dc.SelectBrush();
  dc.SelectPen(PS_DOT,1,cBLOCK);
	dc.SetROP2(R2_COPYPEN);

  dc.Rectangle(theArea);
}


CDrawBlockExport::CDrawBlockExport(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
  segment=1;
  a=b=CPoint(0,0);
}

void CDrawBlockExport::EndEdit()
{
  if (!segment) {
	m_pDesign->UnSelect();
	Display();
  }
}

void CDrawBlockExport::Place(CPoint p, CPoint)
{
  if (segment) {
	a=p;
	b=p;
	segment=0;
	Display();
  } else {
	Display();
	b=p;
	Display();
	// Now select all the objects in this rectangle
	m_pDesign->Select(a,b);
	if (m_pDesign->IsSelected())
		m_pDesign->Save(TRUE,TRUE);
	Display();
	segment=1;
	a = b;
	m_pDesign->UnSelect();
  }
}


void CDrawBlockExport::Move(CPoint p)
{
  if (!segment) {
	Display();
	b=p;
	Display();
  }

}



BOOL CDrawBlockExport::End()
{
  BOOL r = (m_pDesign->GetSelect()!=NULL);

  // If necessary remove the line from the screen
  if (!segment) {
	Display();
	m_pDesign->UnSelect();
	segment=1;
	a = b;
  }

  return r;
}





////// The block import object //////


CDrawBlockImport::CDrawBlockImport(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
  segment=1;

  // Has the import data been selected yet?
  if (!m_pDesign->IsSelected()) 
  {
	m_pDesign->SelectObject(new CDrawEditItem(pDesign) );
	m_pDesign->LoadAll(1);
  }

  
  if (m_pDesign->IsSelected()) {
	CDrawingObject *obj=m_pDesign->GetHead();

	a=b=CPoint(0,0);
	while (obj!=NULL) {
		b.x = max(b.x,max(obj->a.x,obj->b.x));
		b.y = max(b.y,max(obj->a.y,obj->b.y));
		obj->Display();

		if (obj==m_pDesign->GetSelect())
			break;
		obj=obj->next;
	}
	segment = 0;
  }

}


void CDrawBlockImport::EndEdit()
{
  End();
}


void CDrawBlockImport::Place(CPoint p, CPoint)
{

  if (!segment) {
	m_pDesign->UnSelect();
	segment=1;
 }
}


void CDrawBlockImport::Move(CPoint p)
{
  if (!segment) {
  	CPoint r;

  	r.x=p.x-b.x;
  	r.y=p.y-b.y;

	if (r.x!=0 || r.y!=0) {
		m_pDesign->SelectMove(r);
		b=p;
	}
  }
}



BOOL CDrawBlockImport::End()
{
  // If necessary remove the line from the screen
  if (!segment) {
	m_pDesign->SelectDelete();
	m_pDesign->Invalidate();
	segment=1;
	a = b;
  }

  return FALSE;
}



