/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



// This handles the bus objects

#include "stdafx.h"
#include "TinyCadView.h"
#include "MainFrm.h"
#include "diag.h"
#include "colour.h"
#include "option.h"

#define theWnd ((CMainWindow)dc.theWindow)



////// The BusSlash Class //////

#define SLASH_SIZE 20

void CDrawBusSlash::Load(CArchive &archive)
{
  a = ReadPoint(archive);
  archive >> theDir;
  b=a;
  segment=0;
}

CDrawBusSlash::CDrawBusSlash(CTinyCadDoc *pDesign, int NewDir)
: CDrawingObject( pDesign )
{
	theDir = NewDir;
	a=b=CPoint(0,0);
	segment=1;
	Undo_Level=0;
	Delete_Level=-1;
}

void CDrawBusSlash::Save(CArchive &archive)
{
  WritePoint(archive,a);
  archive << theDir;
}


int CDrawBusSlash::IsInside(int left,int right,int top,int bottom)
{
  CPoint bl;

  if (theDir == 0)
	bl=CPoint(a.x-SLASH_SIZE,a.y+SLASH_SIZE);
  else
	bl=CPoint(a.x-SLASH_SIZE,a.y-SLASH_SIZE);

  return !( (a.x<left && bl.x<=left) || (a.x>right && bl.x>=right)
        || (a.y<top && bl.y<=top) || (a.y>bottom && bl.y>=bottom));
}


ObjType CDrawBusSlash::GetType()
{
  return xBusSlash;
}

int CDrawBusSlash::getMenuID() 
{ 
	switch (theDir)
	{
	case 0:
		return IDM_BUSSLASH; 
	default:
		return IDM_BUSBACK;
	}
	
}


CString CDrawBusSlash::GetName()
{
  return "Bus Entry";
}

void CDrawBusSlash::Move(CPoint p)
{
  Display();
  a=p;
  segment=0;
  Display();
}

void CDrawBusSlash::Place(CPoint p, CPoint)
{
  Display();
  a=p;
  b=p;
  Store();
  Display();	// Write to screen
}


void CDrawBusSlash::Display( BOOL erase )
{
	// Invalidate the symbol
	CRect r( a.x,a.y,b.x,b.y);
	m_pDesign->InvalidateRect( r, erase, SLASH_SIZE * 2 + 2 );
}


void CDrawBusSlash::Paint(Context &dc,paint_options options)
{
  CPoint bl;
  switch (options)
  {
  case draw_selected:
  case draw_handles:
	  dc.SelectPen(PS_SOLID,1,cSELECT);
	  break;
  case draw_selectable:
	  dc.SelectPen(PS_SOLID,1,cPIN_CLK);
	  break;
  default:
	  dc.SelectPen(PS_SOLID,1,cPIN);
  }

  

	dc.SetROP2(R2_COPYPEN);

  if (theDir == 0)
	bl=CPoint(a.x-SLASH_SIZE,a.y+SLASH_SIZE);
  else
	bl=CPoint(a.x-SLASH_SIZE,a.y-SLASH_SIZE);

  dc.MoveTo(bl);
  dc.LineTo(a);

}


// Store the slash in the drawing
void CDrawBusSlash::Store()
{
  CDrawBusSlash *NewObject;

  NewObject = new CDrawBusSlash(m_pDesign);
  *NewObject = *this;

  m_pDesign->Add(NewObject);

  CDrawBusSlash *NewRepeat = new CDrawBusSlash(m_pDesign);

  *NewRepeat = *this;
  (NewRepeat->a).x += m_pDesign->GetRepeatDir().x;
  (NewRepeat->a).y += m_pDesign->GetRepeatDir().y;

  m_pDesign->SetRepeat(NewRepeat);
}



////// The BusName Class //////

BOOL CDrawBusName::CanEdit()
{
  return TRUE;
}

// Look for a seach string in the object
CString CDrawBusName::Find(const char *theSearchString)
{
  CString HoldString;

  HoldString = str;
  HoldString.MakeLower();

  // Now look for the search string in this string
  return (HoldString.Find(theSearchString) != -1) ? str : "";
}

void CDrawBusName::NewOptions()
{
  Display();
  FontStyle = m_pDesign->GetOptions()->GetCurrentFont();
  Display();
}

void CDrawBusName::Save(CArchive &archive)
{
  WritePoint(archive,a);
  archive << dir << str << FontStyle;
}


void CDrawBusName::Load(CArchive &archive)
{
 	a = ReadPoint(archive);
  archive >> dir >> str >> FontStyle;

  // Convert the font number over
  FontStyle = m_pDesign->GetOptions()->GetNewFontNumber(FontStyle);
  CSize size= m_pDesign->GetTextExtent(str, FontStyle );

  if (dir>=2)
	b=CPoint(a.x+size.cx,a.y-size.cy);
  else
	b=CPoint(a.x-size.cy,a.y-size.cx);

  segment=0;
}


ObjType CDrawBusName::GetType()
{
	return xBusName;
}

CString CDrawBusName::GetName()
{
	return "BusName";
}


void CDrawBusName::BeginEdit()
{
  g_EditToolBar.m_BusNameEdit.Open(m_pDesign,this);

  m_pDesign->GetOptions()->SetCurrentFont(FontStyle);
}


void CDrawBusName::EndEdit()
{
  End();
  g_EditToolBar.m_BusNameEdit.Close();
}



void CDrawBusName::Move(CPoint p)
{
  Display();			
  a=p;
  segment=0;
  Display();
}

void CDrawBusName::Place(CPoint p, CPoint)
{
  Display();
  a=p;
  Store();
  Display();
  g_EditToolBar.m_BusNameEdit.ReFocus();
  Display();
}


// The constructor

CDrawBusName::CDrawBusName(CTinyCadDoc *pDesign)
: CDrawingObject( pDesign )
{
	dir=3;
	str="";
	segment=1;
	a=b=CPoint(0,0);
	Undo_Level=0;
	Delete_Level=-1;
    FontStyle = m_pDesign->GetOptions()->GetCurrentFont();
}


void CDrawBusName::Display( BOOL erase )
{
	CRect r( a.x,a.y,b.x,b.y);
	m_pDesign->InvalidateRect( r, erase, 2 );

  if (dir>=2) {
  	r = CRect(b.x+10,a.y-10,b.x-10,a.y+10);
  } else {
   	r = CRect(a.x-10,b.y-10,a.x+10,b.y+10);
  }
	m_pDesign->InvalidateRect( r, erase, 2 );

}



// Display the text on the screen!
void CDrawBusName::Paint(Context &dc,paint_options options)
{
  dc.SelectFont(*m_pDesign->GetOptions()->GetFont(FontStyle),dir);

  CSize size=dc.GetTextExtent(str);
  if (dir>=2)
	b=CPoint(a.x+size.cx,a.y-size.cy);
  else
	b=CPoint(a.x-size.cy,a.y-size.cx);

  
  switch (options)
  {
  case draw_selected:
  case draw_handles:
	  dc.SelectPen(PS_SOLID, 1 ,cSELECT);
	  dc.SetTextColor(cLABEL);
	  break;
  case draw_selectable:
  	  dc.SelectPen(PS_SOLID, 1 , cBLACK_CLK);
	  dc.SetTextColor(cLABEL_CLK);
	  break;
  default:
	  dc.SelectPen(PS_SOLID, 1 , cBLACK);
	  dc.SetTextColor(cLABEL);
  }

  
  dc.TextOut(str,a,options,dir);

	dc.SetROP2(R2_COPYPEN);

  if (dir>=2) {
  	dc.MoveTo(CPoint(b.x+10,a.y-10));
  	dc.LineTo(CPoint(b.x-10,a.y+10));
  } else {
   	dc.MoveTo(CPoint(a.x-10,b.y-10));
  	dc.LineTo(CPoint(a.x+10,b.y+10));
  }

}


// Store the text in the drawing
void CDrawBusName::Store()
{
  CDrawBusName *NewObject;

  NewObject = new CDrawBusName(m_pDesign);
  *NewObject = *this;

  m_pDesign->Add(NewObject);

  m_pDesign->SetRepeat(NULL);
}


// Rotate this object about a point
void CDrawBusName::Rotate(CPoint p,int ndir)
{
  // Translate this point so the rotational point is the origin
  a = CPoint(a.x-p.x,a.y-p.y);
  b = CPoint(b.x-p.x,b.y-p.y);

  if (ndir!=4) {
	if (dir>=2)
		dir = 0;
	else
		dir = 3;
  }

  // Perfrom the rotation
  switch (ndir) {
	case 2: // Left
		a = CPoint(a.y,-a.x);
		b = CPoint(b.y,-b.x);
		break;		
	case 3: // Right
		a = CPoint(-a.y,a.x);
		b = CPoint(-b.y,b.x);
		break;
	case 4: // Mirror
		a = CPoint(-a.x,a.y);
		b = CPoint(-b.x,b.y);
		break;
  }

  // Re-translate the points back to the original location
  a = CPoint(a.x+p.x,a.y+p.y);
  b = CPoint(b.x+p.x,b.y+p.y);

  CPoint la = a;

  // Adjust for the rotation
  if (dir>=2) {
	a = CPoint(min(la.x,b.x),max(la.y,b.y));
	b = CPoint(max(la.x,b.x),min(la.y,b.y));
  } else {
	a = CPoint(max(la.x,b.x),max(la.y,b.y));
	b = CPoint(min(la.x,b.x),min(la.y,b.y));
  }
}


void CDrawBusName::TagResources()
{
 	m_pDesign->GetOptions()->TagFont(FontStyle);
}



