/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include <ctype.h>
#include "TinyCadView.h"
#include "registry.h"
#include "colour.h"
#include "TinyCad.h"



////// The symbol rename dialog //////


BOOL CDlgSymbolRename::OnInitDialog()
{
  SetDlgItemText(SYMBOLRENAME_OLD,*name);
  SetDlgItemText(SYMBOLRENAME_NEW,*name);

  return TRUE;
}

void CDlgSymbolRename::OnOK()
{
  // Change the symbol name
  GetDlgItemText(SYMBOLRENAME_NEW,name->GetBuffer(STRLEN),STRLEN);
  name->ReleaseBuffer();

  EndDialog(IDOK);
}

void CDlgSymbolRename::OnCancel()
{
  EndDialog(IDCANCEL);
}



////// The Update Dialog Box //////
BOOL CDlgUpdateBox::OnInitDialog()
{
  SetDlgItemText(UPDATE_LIBFILE, NameLength((NewSymbol->GetLibName()),30));
  SetDlgItemText(UPDATE_NAME, NewSymbol->name);
  SetDlgItemText(UPDATE_REFERENCE, NewSymbol->reference);
  SetDlgItemText(UPDATE_DESCRIPTION, NewSymbol->description);
  SetDlgItemInt(UPDATE_PPP,NewSymbol->ppp);

  return TRUE;
}


BEGIN_MESSAGE_MAP( CDlgUpdateBox, CDialog )
END_MESSAGE_MAP()

void CDlgUpdateBox::OnOK()
{
  char hold[STRLEN];

  GetDlgItemText(UPDATE_NAME, hold, sizeof(hold));
  NewSymbol->name=hold;

  GetDlgItemText(UPDATE_REFERENCE, hold,sizeof(hold));
  NewSymbol->reference=hold;

  GetDlgItemText(UPDATE_DESCRIPTION, hold,sizeof(hold));
  NewSymbol->description=hold;

  NewSymbol->ppp=GetDlgItemInt(UPDATE_PPP);

  // Only end dialog if it is ok
  if (NewSymbol->IsValid())
	EndDialog(IDOK);
  else
	Message(IDS_BADDETAILS,MB_ICONINFORMATION);
}







////// The dialog box for selecting which device is being edited //////

BOOL CDlgWhichDevice::OnInitDialog()
{
  char Buffer[2];
  Buffer[0] = 'A'+device;
  Buffer[1] = 0;
  SetDlgItemText(SELECTDEVICE_TEXT,Buffer);

  return TRUE;
}

void CDlgWhichDevice::OnOK()
{
  char Buffer[10];

  GetDlgItemText(SELECTDEVICE_TEXT,Buffer,sizeof(Buffer));

  Buffer[0]=toupper(Buffer[0]);

  // Is this a correct device letter?
  if (Buffer[1]!=0 || Buffer[0]<'A' || Buffer[0]>'Z')
	Message(IDS_BADDEVICE,MB_ICONEXCLAMATION);
  else {
	device = Buffer[0]-'A';
	EndDialog( IDOK );
  }
}





////// The library box dialog //////

#define FIELD_LENGTH	35

CDlgLibraryBox::CDlgLibraryBox(CWnd *pParentWnd) 
	: CDialog( IDD_LIBRARY, pParentWnd ) 
{ 
	//{{AFX_DATA_INIT(CDlgLibraryBox)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CDlgLibraryBox::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgLibraryBox)
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP( CDlgLibraryBox, CDialog )
	//{{AFX_MSG_MAP(CDlgLibraryBox)
	ON_COMMAND( LIBRARYBOX_ADD, OnAdd )
	ON_COMMAND( LIBRARYBOX_REMOVE, OnRemove )
	ON_COMMAND( LIBRARYBOX_EDIT, OnEdit )	
	ON_COMMAND( LIBRARYBOX_TIDY, OnTidy )
	ON_LBN_DBLCLK(LIBRARYBOX_AVAILABLE, OnDblclkAvailable)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CDlgLibraryBox::OnInitDialog()
{

  // Set up the list box with the current library names in it
  theListBox = (CListBox *)GetDlgItem(LIBRARYBOX_AVAILABLE);
  theListBox->ResetContent();

  CLibraryFile *pointer=static_cast<CTinyCadApp*>(AfxGetApp())->GetLibHead();

  while (pointer!=NULL) {
	theListBox->AddString(pointer->name);
	pointer=pointer->next;
  }

  return TRUE;
}


void CDlgLibraryBox::OnOK()
{

  // Write the new library details back to the registry
  Registry theRegistry;
  CString theString = "";
  CLibraryFile *pointer = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibHead();

  while (pointer!=NULL) {
	theString += pointer->name;
	if (pointer->next!=NULL)
		theString += ",";
	pointer=pointer->next;
  }
  theRegistry.Place("Libraries",theString);

  EndDialog( IDOK );
}

// Tidy the selected library
void CDlgLibraryBox::OnTidy()
{
  CLibraryFile *lib=LibSelected();

  if (lib==NULL)
	return;

	if (static_cast<CTinyCadApp*>(AfxGetApp())->IsLibInUse( lib ))
	{
		// We cannot tidy whilst in use
		AfxMessageBox( IDS_NOTIDY );
		return;
	}

  // Now edit the library names
  lib->Tidy();
}



// Select a library for editing
void CDlgLibraryBox::OnEdit()
{
  CLibraryFile *lib=LibSelected();

  if (lib==NULL)
	return;

  static_cast<CTinyCadApp*>(AfxGetApp())->editLibrary( lib );

  EndDialog(IDOK);
}


// Remove a library from the list of libraries in use
void CDlgLibraryBox::OnRemove()
{
  int WhichItem;
  char Buffer[STRLEN];

  // Get the item which has been selected
  if ((WhichItem = theListBox->GetCurSel())==LB_ERR)
	return;

  // Now calculate which library this is
  theListBox->GetText(WhichItem,Buffer);

  CLibraryFile *lib = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibHead(),*last=NULL;
  while (lib!=NULL && strcmp(lib->name,Buffer)!=0) {
	last=lib;
	lib=lib->next;
  }

  if (lib==NULL)
	return;
  
  // Now remove this library from the linked list
  if (last==NULL)
	static_cast<CTinyCadApp*>(AfxGetApp())->SetLibHead(lib->next);
  else
  	last->next=lib->next;

  // Now remove this from the list box in the dialog box
  theListBox->DeleteString(WhichItem);

  // Now delete the library
  delete lib;
}


// Add a new library to this list box
void CDlgLibraryBox::OnAdd()
{
  OPENFILENAME ofn;
  char szFile[256], szFileTitle[256];

  strcpy(szFile, "*.idx\0");

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
  ofn.lpstrFilter = "Library Indexes (*.idx)\0*.dsn\0All files (*.*)\0*.*\0\0";
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter = 0;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFileTitle = szFileTitle;
  ofn.nMaxFileTitle = sizeof(szFileTitle);
  ofn.lpstrInitialDir = NULL;
  ofn.lpstrTitle = NULL;
  ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = "idx";

  if (!GetOpenFileName(&ofn))
       	return;

  // Remove the extension  
  for (int lp=0;szFile[lp]!=0 && szFile[lp]!='.'; lp++) ;
  szFile[lp]=0;

  // Create the library
  CLibraryFile *NewLib = new CLibraryFile;
  NewLib->Attach(szFile);

  // Now add it to the library linked list
  NewLib->next=static_cast<CTinyCadApp*>(AfxGetApp())->GetLibHead();
  static_cast<CTinyCadApp*>(AfxGetApp())->SetLibHead(NewLib);

  theListBox->AddString(NewLib->name);
}


// Find out which of the libraries has been selected
CLibraryFile *CDlgLibraryBox::LibSelected()
{
  int WhichItem;
  char Buffer[STRLEN];
 
  // Get the item which has been selected
  if ((WhichItem = theListBox->GetCurSel())==LB_ERR)
	return NULL;

  // Now calculate which library this is
  theListBox->GetText(WhichItem,Buffer);

  CLibraryFile *lib = static_cast<CTinyCadApp*>(AfxGetApp())->GetLibHead();
  while (lib!=NULL && strcmp(lib->name,Buffer)!=0)
	lib=lib->next;

  return lib;
}


void CDlgLibraryBox::OnDblclkAvailable() 
{
	OnEdit();
}
