/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// This file contains all the revision information for TinyCAD



///////////////////////////////////////////////////////////////////////////////
//
// Nov. 1994	Version 0.95
//		First release! Beta test version.
//
// Jan. 1995	Version 1.00
//		Full release version.
//		Added:		Toolbar
//					Drag 'n' Drop, "File Manager" Associations
//					Undo/Redo
//
//		Bug Fixes:	Edit Item, Import
//
//
// Apr. 1995	Version 1.20			( Release version)
//		Multi-layer version
//		Added:		New File revision number
//					Find/Tags
//					Font style, line style
//					Bus Name
//					CMainWindow destructor (version 1.20a)
//					Win32s support:
//						Arc segments now use Arc rather than PolyBezier
//						Use Tinycad.ini instead of Registry when in Win32s
//
//		Bug Fixes:	Printing, Names Dialog, IsInside Text
//
//
// May 1995		Version 1.21		 	(for Massimo only!)
//		Added:		Win16 Support
//					Line colour and style selection
//					Layers
//					Rulers
//		Bug Fixes:	crash on exit caused by "last got symbol" in library
//
// Sept 1995	Version 1.25			(release version)
//		Added		Text file support
//
// Nov  1995	Version 1.26
//		Added		Clipboard support
//					Drag select in EditItem
//
// Feb  2002	Version 1.50
//		Yes, this really is 7 years later!!!
//		Many, many changes in preparation for re-release.
//
//		Changed:	Renamed classes so that they all are in the Cxxx format
//					Drawing now not based on XOR
//					Stripped out use of global variable "thisDesign"
//						- added the m_pDesign variable into the CDrawingObject classes
//					Stripped out Win32s and Win16 support (not required these days...)
//					Separated Transform into Transform & Transform Snap
//

///////////////////////////////////////////////////////////////////////////////
// The following files are used in this compilation:
//
// main.h		- defines the CMainWnd class
// main.cpp  	- defines some of the misc procedures for CMainWnd
// paint.cpp	- defines the OnPaint, OnVScroll and OnHScroll functions
// menu.cpp		- defines all the menu response methods
// design.h 	- defines the Design class
// design.cpp	- defines all the design methods
//
// object.h		- defines all the Object classes
// anotate.cpp	- defines the default object functions, anotation objects, and no-draw objects
// scheme.cpp	- defines the schematic objects
// block.cpp	- defines the block edit objects
// power.cpp    - defines the power object and pin object
// method.cpp	- defines the method object (a composite object)
// item.cpp		- defines the edit item and delete item objects
// bus.cpp		- defines the bus names and bus slashes
// colour.h		- defines the colours of each object and default font sizes
//
// diag.h		- defines the classes for all the non-modal dialog boxes
// diag.cpp		- defines the methods for the non-modal dialog boxes
//
// library.cpp	- defines the library operations for the library class
// symbol.cpp	- defines the operation for the symbol and hash classes
// libdiag.cpp	- defines the dialog boxes for use with the library classes
// library.h	- defines symbols, hash tables, and library objects
//
// registry.cpp - defines the interface to the registry database
// registry.h	- the interface for the above
//
// special.cpp	- defines the auto-anotate, parts list special functions
// net.cpp		- defines the netlist generator and the rules checker
// special.h	- the interface for the above
// net.h		- the interface for the above
//
// cad.rc		- defines the resources used
// resource.h	- defines the macros assigned to each resource
//
// startup.cpp	- defines the class to display the startup window
// startup.h	- the interface for the above
//
// toolbar.cpp	- defines the toolbar menu scheme
// toolbar.h	- the interface for the above
//
// cad.hpj		- help project file
// mh.bat		- compiles the help file
// intro.rtf	- uncompiled help file in rich text format
// menu.rtf		- uncomplied help file for menu help
// toolbar.rtf	- uncompiled help file for toolbar help
// keypress.rtf	- uncompiled help file for accelerator keys help
//
// revision.h	- this file, collects together all gobal revision information



#ifndef __REVISION_H__
#define __REVISION_H__


#define NAME				"TinyCAD"
#define	VERSION				"1.50"

#define MAX_UNDO			100					// The maximum number of Undo's stored

#define FILE_REVISION		4					// The revision number of the file format it saves
#define FILE_MAGIC			0xC5A				// The magic number to spot TinyCAD files
#define REG_REVISION 		"1x20"				// The revision number for storage in the registry database
#define CLIPBOARD_FORMAT	"TinyCAD Design"	// The clipboard name for a tinycad copy
#define CLIPBOARD_SY_FORMAT	"TinyCAD Symbol"	// The clipboard name for a tinycad library copy

#define PIXELSPERMM			5					// How many pixels in each milimetre
#define NormalGrid			100					// The normal grid spacing (in pixels)
#define FineGrid			50					// The fine grid spacing (in pixels)

#endif
