/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "stdafx.h"
#include "TinyCadView.h"
#include "special.h"
#include "library.h"
#include "TinyCad.h"

////// The auto anotate special function //////

AnotateSetup theASetup;

// Auto anotate the design
void CTinyCadView::OnSpecialAnotate()
{
  CDlgAnotateBox theDialog(this,theASetup);

  // Get rid of any drawing tool
  GetDocument()->SelectObject(new CDrawEditItem(GetDocument()));

  // Do the dialog
  if (theDialog.DoModal()!=IDOK)
	return;
  theASetup = theDialog.v;

  // Set the busy icon
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_WAIT ) );

  // Sort the objects into order
  while (GetDocument()->Sort());
  

  // Now add/remove references
  CDrawingObject *pointer = GetDocument()->GetHead();
  CDrawMethod *thisMethod;
  CSymbolRecord *thisSymbol;
  CSymbolRecord *lastSymbol = NULL;
  int value,part=0;
  BOOL IsSet,IsMatch,MissingSymbols=FALSE;
  
  while (pointer!=NULL) {
	// Look for method objects
	if (pointer->GetType() == xMethodEx2) {
		thisMethod = (CDrawMethod *)pointer;
		thisSymbol = thisMethod->GetSymbolData();
		// If there is no symbol then cannot modify this symbol!
		if (thisMethod->IsNoSymbol()) {
			MissingSymbols = TRUE;
			pointer=pointer->next;
			continue;
		}

		// Has this symbol got a reference?
		IsSet   = strchr(thisMethod->GetRef(),'?')==NULL;
		IsMatch = (theASetup.reference!=2) || (theASetup.matchval == thisSymbol->reference);

		// Do we need to add or remove this symbol?
		if (theASetup.action == 0 && IsMatch) {
			// Add a reference
			// Was the last symbol reference the same as this one?
			if ((lastSymbol==NULL) || ((lastSymbol->reference) != (thisSymbol->reference))) {
				if (theASetup.reference != 1)
					value = (theASetup.value == 0) ? 1 : theASetup.startval;
				else {
					// Find the highest value for this symbol reference
					CDrawMethod *ObjPtr = thisMethod;
					int maxval = 1;
					while (ObjPtr!=NULL && ObjPtr->GetType()==xMethodEx2
					    && (ObjPtr->GetSymbolData())->reference==thisSymbol->reference) {
						int val = ObjPtr->GetRefVal();
						if (val>maxval)
							maxval = val;
						ObjPtr = (CDrawMethod*) (ObjPtr->next);
					}
					value = maxval+1;
				}
			}
			if (lastSymbol!=thisSymbol && part!=0) {
				part = 0;
				value ++;
			}

			if (theASetup.reference != 1 || !IsSet) {
				thisMethod->SetRefVal(value);
				thisMethod->SetPart(part);
				if (++part == thisSymbol->ppp) {
					value++;
					part = 0;
				}
			}
		} else if (IsMatch) {
			// Remove the references
			if (IsSet && (theASetup.value!=1 || thisMethod->GetRefVal()>=theASetup.startval) ) {
				thisMethod->SetRef(thisSymbol->reference);
				thisMethod->SetPart(0);
			}
		}

		lastSymbol = thisSymbol;
	}

	pointer=pointer->next;
  }


  // Where there any missing symbols?
  if (MissingSymbols)
	Message(IDS_MISSMETH,MB_ICONEXCLAMATION);


  // Restore the correct cursor
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );


  // Ensure the window is re-drawn
  GetDocument()->SetModifiedFlag( TRUE );
  Invalidate();  
}



// The Anotate Setup values class
AnotateSetup::AnotateSetup()
{
  theASetup.action=0;
  theASetup.reference=0;
  theASetup.value=0;
  theASetup.matchval="U?";
  theASetup.startval=1;
}

// The anotate box dialog

BOOL CDlgAnotateBox::OnInitDialog()
{
  SetDlgItemText(ANNOTATEBOX_MATCHVAL,v.matchval);
  SetDlgItemInt(ANNOTATEBOX_STARTVAL,v.startval);
  CheckRadioButton(ANNOTATEBOX_ADD,ANNOTATEBOX_REMOVE,ANNOTATEBOX_ADD+v.action);
  CheckRadioButton(ANNOTATEBOX_ALL,ANNOTATEBOX_MATCH,ANNOTATEBOX_ALL+v.reference);
  CheckRadioButton(ANNOTATEBOX_DEF,ANNOTATEBOX_START,ANNOTATEBOX_DEF+v.value);

  return TRUE;
}


void CDlgAnotateBox::OnOK()
{
  v.action = GetCheckedRadioButton(ANNOTATEBOX_ADD,ANNOTATEBOX_REMOVE)-ANNOTATEBOX_ADD;
  v.reference = GetCheckedRadioButton(ANNOTATEBOX_ALL,ANNOTATEBOX_MATCH)-ANNOTATEBOX_ALL;
  v.value = GetCheckedRadioButton(ANNOTATEBOX_DEF,ANNOTATEBOX_START)-ANNOTATEBOX_DEF;

  char hold[256];
  GetDlgItemText(ANNOTATEBOX_MATCHVAL,hold,sizeof(hold));
  v.matchval = hold;
  v.startval = GetDlgItemInt(ANNOTATEBOX_STARTVAL);

  EndDialog( IDOK );
}

BEGIN_MESSAGE_MAP( CDlgAnotateBox, CDialog )
	ON_EN_CHANGE(ANNOTATEBOX_STARTVAL, OnChangeStart)
END_MESSAGE_MAP()



void CDlgAnotateBox::OnChangeStart()
{
  CheckRadioButton(ANNOTATEBOX_DEF,ANNOTATEBOX_START,ANNOTATEBOX_START);
}




////// The Parts List (Bill of Materials) special function //////


#define PARTSLIST_PAD	25
// Increment the xpos counter
void XInc(FILE *theFile,int &Xpos,int amount)
{
  Xpos += amount;

  if (Xpos >=PARTSLIST_PAD) {
	fprintf(theFile,"\n");
	Xpos = amount;
  }
}

void CTinyCadView::OnSpecialBom()
{
  // Get rid of any drawing tool
  GetDocument()->SelectObject(new CDrawEditItem(GetDocument()));

  // Get the file name for the parts list
  OPENFILENAME ofn;
  char szFile[256], szFileTitle[256];

  if (GetDocument()->GetNamed()) {
	strcpy(szFile,GetDocument()->GetName());
	for (int lp=0;szFile[lp]!=0 && szFile[lp]!='.';lp++) ;
	strcpy(szFile+lp, ".pts");
  } else
	strcpy(szFile,"Parts.pts");

  strcpy(szFileTitle,szFile);

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = this->m_hWnd;
  ofn.lpstrFilter = "Parts List (*.pts)\0*.pts\0All files (*.*)\0*.*\0\0";
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter = 0;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFileTitle = szFileTitle;
  ofn.nMaxFileTitle = sizeof(szFileTitle);
  ofn.lpstrInitialDir = NULL;
  ofn.lpstrTitle = NULL;
  ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST  | OFN_NOCHANGEDIR;
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = "pts";
  ofn.lpstrTitle = "Create Parts List";

  if (!GetSaveFileName(&ofn))
      	return;


  FILE *theFile = fopen(szFile,"w");
  if (theFile == NULL) {
	Message(IDS_CANNOTOPEN);
	return;
  }

  // Set the Busy icon
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_WAIT ) );

  while (GetDocument()->SortName()) ;

  CDrawingObject *ObjPtr = GetDocument()->GetHead();


  fprintf(theFile,"====+  Parts List for %s  +====\n\n",(const char *)(GetDocument()->GetName()));

  BOOL MissingRef = FALSE;
  while (ObjPtr!=NULL) {
	if (ObjPtr->GetType() == xMethodEx2) {
		int Xpos = 0, peices=0;

		CDrawMethod *thisMethod = (CDrawMethod*)ObjPtr;
		CString name = thisMethod->GetName();

		// Write the references
		CString LastRef = "";
		while (thisMethod!=NULL && thisMethod->GetName() == name) {
			// Do not write out references more than once
			if (thisMethod->GetRef() != LastRef) {
				if (thisMethod != ObjPtr && Xpos!=PARTSLIST_PAD-1) {
					XInc(theFile,Xpos,1);
					fprintf(theFile,",");
				}
				XInc(theFile,Xpos,(thisMethod->GetRef()).GetLength());
				fprintf(theFile,"%s",(const char *)(thisMethod->GetRef()));
				peices ++;
			}
			if (thisMethod->GetRef()==(thisMethod->GetSymbolData())->reference)
				MissingRef = TRUE;
			LastRef = thisMethod->GetRef();
			if ((thisMethod->next) != NULL && (thisMethod->next)->GetType()==xMethodEx2)
				thisMethod = (CDrawMethod *)(thisMethod->next);
			else	
				thisMethod = NULL;
		}

		// Pad with spaces
		while (Xpos < PARTSLIST_PAD) {
			fprintf(theFile," ");
			Xpos ++;
		}

		// Write the name
		fprintf(theFile,"   %3d   %s\n",peices,(const char *)name);

		ObjPtr = thisMethod;
	} else
		ObjPtr = ObjPtr->next;
  }

  fclose(theFile);

  // Restore the normal cursor
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );

  // Where there any errors?
  if (MissingRef)
	Message(IDS_MISSREF);
}




////// Tidy the design //////


void DelObject(CTinyCadDoc *pDesign, CDrawingObject *pointer,CDrawingObject *last)
{
  if (last == NULL)
	pDesign->SetHead(pointer->next);
  else
	last->next = pointer->next;

  delete pointer;
}


// Auto anotate the design
void CTinyCadView::OnSpecialTidy()
{
  // Get rid of any drawing tool
  GetDocument()->SelectObject(new CDrawEditItem(GetDocument()));

  // Set the busy icon
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_WAIT ) );

  // Search design for:
  // - zero length wires, lines, dashed lines, buses
  // - zero size rectangles, ellipeses,arcs
  // - zero length strings, labels

  CDrawingObject *pointer = GetDocument()->GetHead(), *lastptr = NULL, *nextptr;

  while (pointer!=NULL) {
	nextptr = pointer->next;
	switch (pointer->GetType()) {
		case xWire:
		case xLineEx2:
		case xDash:
		case xBus:
			if (pointer->a == pointer->b)
				DelObject(GetDocument(),pointer,lastptr);
			break;
		case xCircleEx2:
		case xSquareEx2:
		case xArcEx2:
			if (pointer->a == pointer->b)
				DelObject(GetDocument(),pointer,lastptr);
			break;
		case xTextEx:
		case xLabelEx:
			if (((CDrawText *)pointer)->GetValue()=="")
				DelObject(GetDocument(),pointer,lastptr);
			break;
	}
	lastptr = pointer;
	pointer = nextptr;
  }


  GetDocument()->SetModifiedFlag( TRUE );

  // Restore the correct cursor
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );

  Message(IDS_DESTIDIED);
}

