/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __SYMBOL_H__
#define __SYMBOL_H__

class CTinyCadDoc;
class CLibraryFile;
class CTinyCadSymbolDoc;
class CDrawingObject;

// Here is the data that is associated with
// all symbols
class CSymbolRecord
{
protected:
	CDrawingObject *LoadSymbol( CTinyCadDoc *pDesign, CArchive &theArchive, CPoint &tr );

public:

	CString	name;			// The name of this symbol
	CString	reference;		// The default reference
	CString description;	// The description of this symbol
	BYTE 	ppp;			// Parts per package
};


class CDesignFileSymbol : public CSymbolRecord
{
protected:
	CTinyCadDoc *m_pDesign;
	CDrawingObject *m_pMethod;
	CPoint	m_Tr;

public:
	CDesignFileSymbol(CTinyCadDoc *pDesign, CDrawingObject *pMethod, CPoint tr );
	virtual ~CDesignFileSymbol();		// The Destructor
	
	void Save(CArchive &);		// Save this symbol into an archive
	void Load(CTinyCadDoc *, CArchive &);		// Load this symbol from an archive

	// Return the methods object
	virtual CDrawingObject *GetMethod( CPoint &tr );

	CTinyCadDoc* GetDesign() { return m_pDesign; }
};


class CLibraryFileSymbol : public CSymbolRecord
{
private:
	CDrawingObject *GetMethod( CTinyCadDoc *pDesign, CPoint &tr );	

public:
	DWORD	FilePos;		// The position in the file of this symbol

	CLibraryFile *lib;		// The library that this symbol comes from

	CLibraryFileSymbol(CLibraryFile *);	// The constructor
	CLibraryFileSymbol();				// Another constructor...
	virtual ~CLibraryFileSymbol();		// The Destructor

	
	void Save(CArchive &);		// Save this symbol into an archive
	void Load(CArchive &);		// Load this symbol from an archive
	void OldLoad1(CArchive &);	// Load the old versions of symbol
	void OldLoad2(CArchive &);	// Load the old versions of symbol

	BOOL IsValid();

	CString Find(const char *);

	// Return the methods object
	CString GetLibName();

	CDesignFileSymbol *GetDesignSymbol( CTinyCadDoc *pDesign );
};


#endif


