/*
	TinyCAD program for schematic capture
	Copyright 1994/1995/2002 Matt Pyne.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "TinyCad.h"

#include "TinyCadDoc.h"
#include "Revision.h"
#include "Option.h"
#include "Library.h"
#include "diag.h"
#include "colour.h"
#include "registry.h" 
#include "TinyCadView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTinyCadDoc

IMPLEMENT_DYNCREATE(CTinyCadDoc, CDocument)

BEGIN_MESSAGE_MAP(CTinyCadDoc, CDocument)
	//{{AFX_MSG_MAP(CTinyCadDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTinyCadDoc construction/destruction

CTinyCadDoc::CTinyCadDoc()
{
  theOptions.Init( this );

  head=NULL;
  SelObj=NULL;
  selectable = NULL;

  named=0;

  MaxXSize = 297 * PIXELSPERMM;
  MaxYSize = 210 * PIXELSPERMM;

  Title = "";
  Author = "";
  Revision = "1.0";
  DocNumber = "";
  Organisation = "";
  Sheets = "1 of 1";
  ShowDetails = TRUE;
  DateStr = CTime::GetCurrentTime().Format("%d %B %Y");

  name = "";

  Undo_Level = 0;
  Top_Level = 0;
  Bottom_Level = 0;
  Undo_Inc = FALSE;
  Redo_List = NULL;

  edit=NULL;
  NextRepeat = NULL;
  LastRepeat = NULL;
  RepeatDir = CPoint(0,1);
  NameDir = 1;
  PinDir = 1;


  part = 0;


  // Now select a drawing object
  SelectObject(new CDrawEditItem(this));

	// Attempt to load the defaults for the grid
	WORD hold;
	Registry theRegistry;

	m_snap.SetAccurateGrid( theRegistry.Get("GridSpacing",&hold) ? hold : NormalGrid );
	m_snap.SetGridSnap( theRegistry.Get("GridSnap",&hold) ? hold : TRUE );
}


CTinyCadDoc::~CTinyCadDoc()
{
	Registry theRegistry;
	theRegistry.Place("GridSpacing",m_snap.GetAccurateGrid());
	theRegistry.Place("GridSnap",m_snap.GetGridSnap());

	CDrawingObject *pointer = GetHead();

	while (pointer != NULL) {
		CDrawingObject *next = pointer->next;
		delete pointer;
		pointer = next;
	}

	SetHead(NULL);
	SetSelect(NULL);

	// Remove the undo/redo list
	pointer = Redo_List;
	while (pointer != NULL) {
		CDrawingObject *next = pointer->next;
		delete pointer;
		pointer = next;
	}


	// Delete the editing tool
	delete edit;
	delete NextRepeat;
	delete LastRepeat;
}

BOOL CTinyCadDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

  MaxXSize = 297 * PIXELSPERMM;
  MaxYSize = 210 * PIXELSPERMM;


  // Has the page size been stored in the registry?
  Registry theRegistry;
  int Fd;
  CString GetString;
  if (theRegistry.Get("PageSize",GetString) && (Fd=GetString.Find(','))!=-1 ) {
	CPoint NewSize;
	NewSize.y = atoi(GetString.Mid(Fd+1));
	NewSize.x = atoi(GetString.Left(Fd));
	SetPageBoundries(NewSize);		
  }


  CDrawingObject *obj=head,*next;

  SetPart(0);

  SelectObject(new CDrawEditItem(this));

  // Remove all objects from memory
  while (obj!=NULL) {
	next=obj->next;

	// Remove this object from memory
	delete obj;

	obj=next;
	head=next;
  }
  SelObj=NULL;

  // Now delete the Redo List
  obj = Redo_List;
  while (obj!=NULL) {
	next=obj->next;

	// Remove this object from memory
	delete obj;

	obj=next;
	head=next;
  }
  Undo_Level = 0;
  Top_Level = 0;
  Bottom_Level = 0;
  Undo_Inc = FALSE;
  Redo_List = NULL;

  // reset the design
  named=0;
  Title = "";
  Author = "";
  Revision = "1.0";
  DocNumber = "";
  Organisation = "";
  Sheets = "1 of 1";

  head=NULL;
  SelObj=NULL;

  named=0;


  name = "";

  RepeatDir = CPoint(0,1);
  NameDir = 1;
  PinDir = 1;

  part = 0;

  return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CTinyCadDoc serialization

void CTinyCadDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		Save( ar, TRUE, FALSE );
	}
	else
	{
		Load(ar, TRUE);
		SelObj=NULL;
		SetModifiedFlag( FALSE );
	}
}

/////////////////////////////////////////////////////////////////////////////
// CTinyCadDoc diagnostics

#ifdef _DEBUG
void CTinyCadDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CTinyCadDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTinyCadDoc commands




// Convert this document into a metafile for pasting into
// other applications
HENHMETAFILE CTinyCadDoc::CreateMetafile( CDC &ref_dc )
{

	// Calculate the boundries
	CRect rect = CRect(0,0,0,0);

	if (GetSelect() != NULL)
	{
		CDrawingObject *pointer = GetHead();
		if (pointer != NULL) 
		{
			rect.top = pointer->a.y;
			rect.bottom = pointer->a.y;
			rect.left = pointer->a.x;
			rect.right = pointer->a.x;
		}
		while (pointer != NULL) 
		{
			rect.top 	 = min(rect.top,min(pointer->a.y,pointer->b.y));
			rect.bottom = max(rect.bottom,max(pointer->a.y,pointer->b.y));
			rect.left 	 = min(rect.left,min(pointer->a.x,pointer->b.x));
			rect.right  = max(rect.right,max(pointer->a.x,pointer->b.x));

			if (pointer == GetSelect())
				break;
			pointer = pointer->next;
		}
	}
	else
	{
		rect = CRect( 0,0, MaxXSize, MaxYSize );
	}

	rect.InflateRect( 0,0, 10,10 );


	// Transform the origin so the design lies at 0,0
	Transform newTransform;


	int iWidthMM = ref_dc.GetDeviceCaps(HORZSIZE); 
	int iHeightMM = ref_dc.GetDeviceCaps(VERTSIZE); 
	int iWidthPels = ref_dc.GetDeviceCaps(HORZRES); 
	int iHeightPels = ref_dc.GetDeviceCaps(VERTRES); 
 
	// Convert client coordinates to .01-mm units. 
	// Use iWidthMM, iWidthPels, iHeightMM, and 
	// iHeightPels to determine the number of 
	// .01-millimeter units per pixel in the x- 
	//  and y-directions. 
 
	rect.left = (rect.left * iWidthMM * 100)/iWidthPels; 
	rect.top = (rect.top * iHeightMM * 100)/iHeightPels; 
	rect.right = (rect.right * iWidthMM * 100)/iWidthPels; 
	rect.bottom = (rect.bottom * iHeightMM * 100)/iHeightPels;


	// Create the metafile device context. 
	HDC hdcMeta = CreateEnhMetaFile(ref_dc.m_hDC,NULL, &rect, Title); 
 
	if (!hdcMeta) 
	{
		AfxMessageBox("Cannot create enhanced metafile for the copy");
		return NULL;
	}

	CDC theMetaDC;
	theMetaDC.Attach( hdcMeta );

	// Now render into this dc
	newTransform.SetZoom( 100 );

	// Create the DC and render...
	{
		Context dc(&theMetaDC,newTransform);

		// Draw the design into the metafile
		CDrawingObject *pointer = GetHead();
		while (pointer != NULL) 
		{
			pointer->Paint(dc,draw_normal);
			if (pointer == GetSelect())
				break;
			pointer = pointer->next;
		}


		if (GetSelect() == NULL)
		{
			// Show the design details
			DisplayDetails( dc );
		}

	}

	theMetaDC.Detach();

	return CloseEnhMetaFile( hdcMeta );
}


// Update the page boundries etc, using a printer device context
void CTinyCadDoc::SetPageBoundries(PRINTDLG &pd)
{
  CDC *dc = CDC::FromHandle( pd.hDC );

  // Change the scaling to print correct size
  MaxXSize=dc->GetDeviceCaps(HORZSIZE)*PIXELSPERMM;
  MaxYSize=dc->GetDeviceCaps(VERTSIZE)*PIXELSPERMM;
}


// Make all symbol aware of the fact the libraries have changed
void CTinyCadDoc::ResetSymbols()
{
	theOptions.ResetSymbols();
}


// Tag all the resources being used by this design
void CTinyCadDoc::TagAllResources()
{
  CDrawingObject *pointer=GetHead(),*last = NULL;

  while (pointer!=NULL) {
  	pointer->TagResources();
	pointer=pointer->next;	
  }

}

void CTinyCadDoc::SetScrollCentre(CPoint q)
{ 
// !!!	owner->SetScrollCentre( q ); 
}

// Remove all errors from this design
void CTinyCadDoc::DeleteErrors()
{
  // Get rid of any drawing tool
  SelectObject(new CDrawEditItem(this));

  CDrawingObject *pointer=GetHead(),*last = NULL, *next;

  while (pointer != NULL) {
  	next = pointer->next;

	if (pointer->GetType()==xError) {
		if (last != NULL)
			last->next = next;
		else
			SetHead(next);

		delete pointer;
	} else
		last = pointer;

	pointer = next;
  }

	Invalidate();
}


// Is this document editing a library?
bool CTinyCadDoc::IsLibInUse( CLibraryFile *lib )
{
	return false;
}


// Add a new object to this drawing
void CTinyCadDoc::Add(CDrawingObject *NewObject)
{
  // Is this a valid object?
  if (NewObject == NULL)
	return;

  // Do we need to increment the Undo Level?
  FlushRedo();

  // Set the Undo Level of the new object
  NewObject->Undo_Level = Top_Level;
  NewObject->Delete_Level = -1;

  // Store the object in the linked list
  NewObject->next=head;
  head=NewObject;

  // Set the dirty flag
  if (NewObject->GetType()!=xError)
	SetModifiedFlag( TRUE );
}

void CTinyCadDoc::SetHead(CDrawingObject *NewHead)
{
	head=NewHead; 
}


void CTinyCadDoc::SetSelect(CDrawingObject *NewSelObj)
{ 
	CDrawingObject *pointer = GetHead();
	while (pointer != NULL && pointer != SelObj)
	{
		pointer->Display( FALSE );
		pointer = pointer->next;
	}

	SelObj=NewSelObj; 

	pointer = GetHead();
	while (pointer != NULL && pointer != SelObj)
	{
		pointer->Display( FALSE );
		pointer = pointer->next;
	}
}



// Undo the last action
void CTinyCadDoc::Undo()
{
  Undo_Level --;
  SwapLists();
  Invalidate();
}

// Redo the last action
void CTinyCadDoc::Redo()
{
  if (Undo_Level<Top_Level) {
	Undo_Level ++;
	SwapLists();
	Invalidate();
  }
}


// Swap non-displayable items into Redo List and displable items from redo
// list into normal list
void CTinyCadDoc::SwapLists()
{
  // Move the objects from the Redo List to the object list
  CDrawingObject *pointer = Redo_List, *next, *last = NULL;
  while (pointer!=NULL) {
	next = pointer->next;

	if (pointer->DoesExist()) {
		if (last!=NULL)
			last->next = pointer->next;
		else
			Redo_List = pointer->next;
		pointer->next = head;
		head = pointer;
	} else 
		last = pointer;
	pointer = next;
  }

  // Move the no longer valid objects onto the Redo List
  pointer = head;
  last = NULL;
  while (pointer!=NULL) {
	next=pointer->next;
	if ( !(pointer->DoesExist()) ) {
		if (last != NULL)
			last->next = pointer->next;
		else
			head = pointer->next;
		pointer->next = Redo_List;
		Redo_List = pointer;
	} else
		last = pointer;

	pointer = next;
  }


}


// Flush the Redo Buffer
void CTinyCadDoc::FlushRedo()
{
  // Delete all objects that MAX_UNDO's ago won't exist!
  int Old_Undo_Level = Undo_Level;
  Undo_Level -= MAX_UNDO;

  // Does the Redo buffer need flushing?
  if (Top_Level!=Undo_Level) {
	// Delete all objects from the Redo list
	CDrawingObject *pointer = Redo_List;
	CDrawingObject *next,*last = NULL;
	while (pointer!=NULL) {
		next=pointer->next;

		if ( pointer->Delete_Level == -1 || pointer->Delete_Level<Undo_Level ) {
			if (last!=NULL)
				last->next = next;
			else
				Redo_List = next;
			delete pointer;
		} else
			last = pointer;

		pointer = next;
	}
  }
  Bottom_Level = max(Bottom_Level,Undo_Level);
  Undo_Level = Old_Undo_Level;

  // Do we need to increment the Undo Level?
  if (Undo_Inc) {
	Undo_Level++;
	Top_Level = Undo_Level;
	Undo_Inc = FALSE;
  }

  // Remove any deletes in the design list
  CDrawingObject *pointer = GetHead();
  while (pointer!=NULL) {
	if (pointer->Delete_Level != -1)
		pointer->Delete_Level = -1;
	pointer = pointer->next;
  }
}


// Perform a single pass of a bubble sort
// Sorting is done on in this order:
// 1. All objects are sorted by object type
// 2. Method objects are sorted by :-
//	2a Default Reference
//	2b Symbol type

#define thisMethod	((CDrawMethod*)ObjPtr)
#define nextMethod	((CDrawMethod*)ObjNext)
#define thisSymbol	(thisMethod->GetSymbolData())
#define nextSymbol	(nextMethod->GetSymbolData())

BOOL CTinyCadDoc::Sort()
{
  CDrawingObject *ObjPtr = GetHead(), *ObjNext = NULL, *last=NULL;
  int again = FALSE;

  // Perform a sinlge pass of a bubble sort
  while (ObjPtr != NULL) {
	ObjNext = ObjPtr->next;
	if (ObjNext!=NULL && ((ObjPtr->GetType() < ObjNext->GetType())
		|| (	(ObjPtr->GetType()==xMethodEx2 && ObjNext->GetType()==xMethodEx2)
		     && ( (thisSymbol->reference > nextSymbol->reference)
		       || (thisSymbol->reference == nextSymbol->reference && thisSymbol > nextSymbol) )))) {
		again = TRUE;
		ObjPtr->next = ObjNext->next;
		ObjNext->next = ObjPtr;
		if (last != NULL)
			last->next = ObjNext;
		else
			SetHead(ObjNext);
		}

	last = ObjPtr;
	ObjPtr = ObjPtr->next;
  }

  return again;
}


// Perform a single pass of a bubble sort
// Sorting is done on in this order:
// 1. All objects are sorted by object type
// 2. Method objects are sorted by :-
//	2a Current Reference

BOOL CTinyCadDoc::SortRef()
{
  CDrawingObject *ObjPtr = GetHead(), *ObjNext = NULL, *last=NULL;
  int again = FALSE;

  // Perform a sinlge pass of a bubble sort
  while (ObjPtr != NULL) {
	ObjNext = ObjPtr->next;
	if (ObjNext!=NULL && ((ObjPtr->GetType() < ObjNext->GetType())
		|| (	 (ObjPtr->GetType()==xMethodEx2 && ObjNext->GetType()==xMethodEx2)
		     &&  thisMethod->GetField(CDrawMethod::Ref) > nextMethod->GetField(CDrawMethod::Ref) ))) {
		again = TRUE;
		ObjPtr->next = ObjNext->next;
		ObjNext->next = ObjPtr;
		if (last != NULL)
			last->next = ObjNext;
		else
			SetHead(ObjNext);
		}

	last = ObjPtr;
	ObjPtr = ObjPtr->next;
  }

  return again;
}


// Perform a single pass of a bubble sort
// Sorting is done on in this order:
// 1. All objects are sorted by object type
// 2. Method objects are sorted by :-
//	If default references are the same then:-
//		- Sort by name, then reference
//	else
//		- Sort by reference, then name

BOOL CTinyCadDoc::SortName()
{
  CDrawingObject *ObjPtr = GetHead(), *ObjNext = NULL, *last=NULL;
  int again = FALSE;

  // Perform a sinlge pass of a bubble sort
  while (ObjPtr != NULL) {
	ObjNext = ObjPtr->next;
	if (ObjNext!=NULL && ((ObjPtr->GetType() < ObjNext->GetType())
		|| (	(ObjPtr->GetType()==xMethodEx2 && ObjNext->GetType()==xMethodEx2)
		     && ( thisSymbol->reference == nextSymbol->reference
		     ?  ( (thisMethod->GetName() > nextMethod->GetName())
		       || (thisMethod->GetName() == nextMethod->GetName() && thisMethod->GetRef() > nextMethod->GetRef()))
		     :  ( (thisMethod->GetRef() > nextMethod->GetRef())
		       || (thisMethod->GetRef() == nextMethod->GetRef() && thisMethod->GetName() > nextMethod->GetName()) ))))) {

		again = TRUE;
		ObjPtr->next = ObjNext->next;
		ObjNext->next = ObjPtr;
		if (last != NULL)
			last->next = ObjNext;
		else
			SetHead(ObjNext);
		}

	last = ObjPtr;
	ObjPtr = ObjPtr->next;
  }

  return again;
}


// Perform a single pass of a bubble sort
// Sorting is done on in this order:
// 1. All objects are sorted by layer

BOOL CTinyCadDoc::SortLayer()
{
  CDrawingObject *ObjPtr = GetHead(), *ObjNext = NULL, *last=NULL;
  int again = FALSE;


  return again;
}



// The object selection functions
// This selects objects in a box, and moves them to the top of the list
void CTinyCadDoc::Select(CPoint p1,CPoint p2)
{
  CDrawingObject *obj=head,*last=NULL;

  int left=Min(p1.x,p2.x);
  int right=Max(p1.x,p2.x);
  int top=Min(p1.y,p2.y);
  int bottom=Max(p1.y,p2.y);

  UnSelect();

  while (obj) {
	if (obj->IsInside(left,right,top,bottom) ) {
		obj->Display();
		if (SelObj==NULL)
			SelObj=obj;
		// Move this object to the head of the list
		if (last!=NULL) {
			last->next=obj->next;
			obj->next=head;
			head=obj;
			obj=last->next;
		} else {
			last=obj;
			obj=obj->next;
		}
	} else {
		last=obj;
	  	obj=obj->next;
	}
  }
}


// Select an object
void CTinyCadDoc::Select(CDrawingObject *selObject)
{
	// Find the parent of this object
	CDrawingObject *last = NULL, *pointer = GetHead();

	while (pointer!=selObject) {
		last = pointer;
		pointer = pointer->next;
	}

	// Remove this object from the linked list
	if (last != NULL)
		last->next = pointer->next;
	else
		SetHead(pointer->next);

	// Now add at head of the list
	pointer->next = GetHead();
	SetHead(pointer);

	// Select the object
	SetSelect(pointer);
}



// Unselect the objects
void CTinyCadDoc::UnSelect()
{
  // Get rid of any currently selected objects
  if (SelObj!=NULL) {
	CDrawingObject *pointer = head;

	while (pointer!=NULL) {
		pointer->Display();
		if (pointer==SelObj)
			break;
		pointer=pointer->next;
	}

	SelObj=NULL;
  }
}

// Delete the selected objects!
void CTinyCadDoc::SelectDelete()
{
  CDrawingObject *obj=head, *next;

  if (SelObj == NULL)
	return;

  FlushRedo();

  while (obj!=NULL) {
	obj->Delete_Level = Undo_Level;

	if (obj==SelObj)
		break;

	obj=obj->next;
  }
  // Now move this list to the Redo list
  next = SelObj->next;
  SelObj->next = Redo_List;
  Redo_List = head;
  head = next;

  // Unselect all objects
  SelObj=NULL;

  SetModifiedFlag( TRUE );
}


// Move the selected items (the CPoint is the relative shift)
void CTinyCadDoc::SelectMove(CPoint r)
{
  CDrawingObject *obj=head;

  if (SelObj==NULL)
	return;

  // Remove all the objects from the screen
  while (obj!=NULL) {
	// Change the object to XOR
	obj->Display();
	// Move the object
	obj->a.x += r.x;
	obj->a.y += r.y;
	obj->b.x += r.x;
	obj->b.y += r.y;
  	if (obj==SelObj)
		break;
	obj=obj->next;
  }
  
  obj=head;
  while (obj!=NULL) {
	// Change the object to XOR
	obj->Display();
	if (obj==SelObj)
		break;
	obj=obj->next;
  } 
}



// Place the Selected objects into the Undo List in the original position
void CTinyCadDoc::SelectUndoMove(CPoint r)
{

  if (SelObj == NULL)
	return;

  CDrawingObject *old = GetHead();

  // Make a copy of the selected objects (so as to perform add/delete operation for Redo List)
  CDrawingObject *pointer = GetHead(), *last = NULL, *next;
  while (pointer!=NULL) 
  {
		pointer->Store();
		if (last == NULL)
			last = GetHead();
		if (pointer==GetSelect())
			break;
		pointer=pointer->next;
  }

  if (last!=NULL)
  {
	pointer = last->next;
  }

  CDrawingObject *NewSelect = pointer;
  while (pointer!=NULL) 
  {
	next = pointer->next;
	last->next = pointer->next;
	pointer->next = GetHead();
	SetHead(pointer);
	if (pointer == GetSelect())
		break;
	pointer=next;
  }
  SetSelect(NewSelect);

  // Now move these objects back to their orignal position and delete them!
  CDrawingObject *obj = GetHead();
  while (obj!=NULL) 
  {
	// Move the object
	obj->a.x += r.x;
	obj->a.y += r.y;
	obj->b.x += r.x;
	obj->b.y += r.y;
  	if (obj==SelObj)
		break;
	obj=obj->next;
  }
  
  SelectDelete();

  // Now select the new objects
  SelObj = last;

  g_EditToolBar.changeSelected( old, GetHead() );


  SetModifiedFlag( TRUE );

}





// Save the current design
BOOL CTinyCadDoc::Save(BOOL GetName,BOOL SaveSelect)
{

  CString theFileName = GetPathName();

  // Get the filename if necessary
  if (GetName) { 
        OPENFILENAME ofn;
        char szFile[256], szFileTitle[256];

	if (named)
		strcpy(szFile,name);
	else
        	strcpy(szFile, "*.dsn\0");

        ofn.lStructSize = sizeof(OPENFILENAME);
        ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
	  	ofn.lpstrFilter = "Designs (*.dsn)\0*.dsn\0All files (*.*)\0*.*\0\0";
        ofn.lpstrCustomFilter = NULL;
        ofn.nMaxCustFilter = 0;
        ofn.nFilterIndex = 1;
        ofn.lpstrFile = szFile;
        ofn.nMaxFile = sizeof(szFile);
        ofn.lpstrFileTitle = szFileTitle;
        ofn.nMaxFileTitle = sizeof(szFileTitle);
        ofn.lpstrInitialDir = NULL;
        ofn.lpstrTitle = NULL;
        ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
        ofn.nFileOffset = 0;
        ofn.nFileExtension = 0;
        ofn.lpstrDefExt = "dsn";

        if (!GetSaveFileName(&ofn))
        	return FALSE;

		theFileName = szFile;

			name = szFileTitle;
			named=1;
  }


  BOOL r;

  // Show the busy icon
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_WAIT ) );


  // Is this file to be saved as text or as a design file?
  	CFile theFile;

  	// Open the file for saving as a CFile for a CArchive
  	r =  theFile.Open(theFileName, CFile::modeCreate | CFile::modeWrite);

	if (r) {
  		// Now save the file
		CArchive ar( &theFile, CArchive::store );
  		Save(ar,TRUE,SaveSelect);

  		SetModifiedFlag( FALSE );
  	}

  // Turn the cursor back to normal
  SetCursor( AfxGetApp()->LoadStandardCursor( IDC_ARROW ) );

  // Was there an error opening the file?
  if (!r)
	// Could not open file to start saving
	Message(IDS_ABORTSAVE,MB_ICONEXCLAMATION);

  return r;
}





// Load a new design
void CTinyCadDoc::LoadAll(BOOL Slect)
{

  OPENFILENAME ofn;
  char szFile[256], szFileTitle[256];

  strcpy(szFile, "*.dsn\0");

  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.hwndOwner = AfxGetMainWnd()->m_hWnd;
  ofn.lpstrFilter = "Designs (*.dsn)\0*.dsn\0All files (*.*)\0*.*\0\0";
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter = 0;
  ofn.nFilterIndex = 1;
  ofn.lpstrFile = szFile;
  ofn.nMaxFile = sizeof(szFile);
  ofn.lpstrFileTitle = szFileTitle;
  ofn.nMaxFileTitle = sizeof(szFileTitle);
  ofn.lpstrInitialDir = NULL;
  ofn.lpstrTitle = NULL;
  ofn.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
  ofn.nFileOffset = 0;
  ofn.nFileExtension = 0;
  ofn.lpstrDefExt = "dsn";

  if (Slect)
	ofn.lpstrTitle = "Import";
  else
	ofn.lpstrTitle = NULL;

  if (!GetOpenFileName(&ofn))
       	return;


  // Do we use the text loading procedure or load from a CArchive?
	  CFile theFile;
	    if (!theFile.Open(szFile, CFile::modeRead)) {
	// Could not open file to start loading
	Message(IDS_ABORTLOAD,MB_ICONEXCLAMATION);
	return;

	CArchive ar( &theFile, CArchive::load );
  	Load(ar,Slect);
	if (!Slect) {
		SelObj=NULL;
		SetModifiedFlag( FALSE );
	}

  }

}





// The methods for the details dialog
BOOL CDlgDetailsBox::OnInitDialog()
{
  SetDlgItemText(DESIGNBOX_FILE,parent->GetPathName());
  SetDlgItemText(DESIGNBOX_TITLE,parent->Title);
  SetDlgItemText(DESIGNBOX_AUTHOR,parent->Author);
  SetDlgItemText(DESIGNBOX_REVISION,parent->Revision);
  SetDlgItemText(DESIGNBOX_DOCUMENT,parent->DocNumber);
  SetDlgItemText(DESIGNBOX_ORGANISATION,parent->Organisation);
  SetDlgItemText(DESIGNBOX_SHEET,parent->Sheets);
  SetDlgItemText(DESIGNBOX_DATE,parent->DateStr);
  CheckDlgButton(DESIGNBOX_DISPLAY, parent->ShowDetails);

  return TRUE;
}

void CDlgDetailsBox::OnOK()
{
  char hold[256];

  GetDlgItemText(DESIGNBOX_TITLE,hold,sizeof(hold));
  parent->Title=hold;
  GetDlgItemText(DESIGNBOX_AUTHOR,hold,sizeof(hold));
  parent->Author=hold;
  GetDlgItemText(DESIGNBOX_REVISION,hold,sizeof(hold));
  parent->Revision=hold;
  GetDlgItemText(DESIGNBOX_DOCUMENT,hold,sizeof(hold));
  parent->DocNumber=hold;
  GetDlgItemText(DESIGNBOX_ORGANISATION,hold,sizeof(hold));
  parent->Organisation=hold;
  GetDlgItemText(DESIGNBOX_SHEET,hold,sizeof(hold));
  parent->Sheets=hold;
  GetDlgItemText(DESIGNBOX_DATE,hold,sizeof(hold));
  parent->DateStr = hold;
  parent->ShowDetails = IsDlgButtonChecked(DESIGNBOX_DISPLAY)!=0;

  Invalidate();

  EndDialog(IDOK);
}


// Redraw our window
void CTinyCadDoc::Invalidate()
{
	UpdateAllViews( NULL, DOC_UPDATE_INVALIDATE, NULL );
}

void CTinyCadDoc::InvalidateRect( CRect r, BOOL erase, int grow )
{
	doc_invalidrect hint;
	hint.r = r;
	hint.grow = grow;
	UpdateAllViews( NULL, erase ? DOC_UPDATE_INVALIDRECTERASE : DOC_UPDATE_INVALIDRECT, &hint );
}

// No obvious answer to this one, but it is
// a function that should go in here...
CSize CTinyCadDoc::GetTextExtent(const char *str, int style )
{
	Transform dummy;
	Context dc(AfxGetMainWnd(), dummy );
	dc.SelectFont(*GetOptions()->GetFont(style),0);
	return dc.GetTextExtent( str );
}


// Set the next repeat object for the editing view
void CTinyCadDoc::SetRepeat( CDrawingObject *pObject )
{
  // Get rid of the last repeat
  if (LastRepeat!=NULL) {
	delete LastRepeat;
  }

  LastRepeat = NextRepeat;

  NextRepeat = pObject;  
}

void CTinyCadDoc::Repeat()
{
  if (NextRepeat!=NULL) 
  {
	NextRepeat->Display();
	NextRepeat->Store();
  }
}

void CTinyCadDoc::SetRepeatDir( int NewDir )
{
  switch(NewDir) {
	case 0: // Up
		RepeatDir = CPoint(0,-1);
		break;
	case 1: // Down
		RepeatDir = CPoint(0,1);
		break;
	case 2: // Left
		RepeatDir = CPoint(-1,0);
		break;
	case 3: // Right
		RepeatDir = CPoint(1,0);
		break;
  }
}


// Select a new editing object
void CTinyCadDoc::SelectObject(CDrawingObject *NewO )
{ 
	if (edit)
	{
		edit->EndEdit();
		delete edit;
	}

	if (NewO)
	{
		NewO->BeginEdit();
	}

  edit=NewO;

  // Increment the undo level
  IncUndoLevel();
}




#define BOXWIDTH	400
#define	LINEHEIGHT	18

// Draw the details box
void CTinyCadDoc::DisplayDetails(Context &dc)
{
  // Do we display the details?
  if (!ShowDetails)
	return;

  // Does the date need to be updated?
  // if (IsModified())
  //  Date = CTime::GetCurrentTime();

  // Select the correct pen
  dc.SelectPen(PS_SOLID,1,cLINE);
  // Select the correct font
  dc.SelectFont(*GetOptions()->GetFont(fTEXT),3);

  // Draw the box to house it all
  CPoint tl = CPoint(MaxXSize - BOXWIDTH-2, MaxYSize - LINEHEIGHT*9 -2);
  CPoint br = CPoint(MaxXSize-2,MaxYSize-2);
  int LineHeight = LINEHEIGHT;
  int TextSpace = LineHeight/2;
  int BottomRow = (br.x-tl.x)/3;
  int MiddleRow = br.x-(br.x-tl.x)/5;

  // Now draw the outline
  dc.MoveTo(tl);
  dc.LineTo(CPoint(br.x,tl.y));
  dc.LineTo(br);
  dc.LineTo(CPoint(tl.x,br.y));
  dc.LineTo(tl);

  // Draw the horizontal lines
  dc.MoveTo(CPoint(tl.x,tl.y+LineHeight*2));
  dc.LineTo(CPoint(br.x,tl.y+LineHeight*2));
  dc.MoveTo(CPoint(tl.x,tl.y+LineHeight*5));
  dc.LineTo(CPoint(br.x,tl.y+LineHeight*5));
  dc.MoveTo(CPoint(tl.x,tl.y+LineHeight*7));
  dc.LineTo(CPoint(br.x,tl.y+LineHeight*7));

  // Draw the vertical lines
  dc.MoveTo(CPoint(MiddleRow,tl.y+LineHeight*5));
  dc.LineTo(CPoint(MiddleRow,tl.y+LineHeight*7));
  dc.MoveTo(CPoint(tl.x+BottomRow,tl.y+LineHeight*7));
  dc.LineTo(CPoint(tl.x+BottomRow,tl.y+LineHeight*9));
  dc.MoveTo(CPoint(MiddleRow,tl.y+LineHeight*7));
  dc.LineTo(CPoint(MiddleRow,tl.y+LineHeight*9));

  // Add the text
  dc.SetTextAlign(TA_LEFT | TA_BOTTOM | TA_NOUPDATECP);
  dc.SetTextColor(cBLACK);
  dc.SetROP2(R2_COPYPEN);
  dc.SetBkMode(TRANSPARENT);

  dc.TextOut(tl.x+TextSpace,tl.y+LineHeight,"Title");
  dc.TextOut(tl.x+TextSpace,tl.y+LineHeight*3,"Author");
  dc.TextOut(tl.x+TextSpace,tl.y+LineHeight*6,"File");
  dc.TextOut(tl.x+TextSpace,tl.y+LineHeight*8,"Revision");
  dc.TextOut(MiddleRow+TextSpace,tl.y+LineHeight*6,"Document");
  dc.TextOut(tl.x+BottomRow+TextSpace,tl.y+LineHeight*8,"Date");
  dc.TextOut(MiddleRow+TextSpace,tl.y+LineHeight*8,"Sheets");

  // Add the actual data!  
  dc.TextOut(tl.x+TextSpace*2,tl.y+LineHeight*2,Title);
  dc.TextOut(tl.x+TextSpace*2,tl.y+LineHeight*4,Author);
  dc.TextOut(tl.x+TextSpace*2,tl.y+LineHeight*5,Organisation);
  dc.TextOut(tl.x+TextSpace*2,tl.y+LineHeight*7,MiddleRow - tl.x - TextSpace * 4, GetPathName());
  dc.TextOut(tl.x+TextSpace*2,tl.y+LineHeight*9,Revision);
  dc.TextOut(MiddleRow+TextSpace*2,tl.y+LineHeight*7,DocNumber);
  dc.TextOut(tl.x+BottomRow+TextSpace*2,tl.y+LineHeight*9,DateStr);
  dc.TextOut(MiddleRow+TextSpace*2,tl.y+LineHeight*9,Sheets);

}


void CTinyCadDoc::SetSelectable(CDrawingObject *obj)
{
	// Only redraw if the selectable changes
	if (obj != selectable)
	{
		if (selectable)
		{
			selectable->Display( FALSE );
		}

		selectable = obj;

		if (selectable)
		{
			selectable->Display( FALSE );
		}
	}
}

CDrawingObject* CTinyCadDoc::GetSelectable()
{
	return selectable;
}

BOOL CTinyCadDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	// Is this a emf file?
	CString path = lpszPathName;
	CString extension;
	int brk = path.ReverseFind('.');
	if (brk != -1)
	{
		extension = path.Mid(brk);
	}
	if (extension.CompareNoCase(".emf") == 0)
	{
		// Load a wmf
		HENHMETAFILE data = (HENHMETAFILE)::GetEnhMetaFile( lpszPathName );
		CDrawMetaFile *pObject = new CDrawMetaFile( this );
		pObject->setMetaFile( data );
		Add( pObject );
	}
	else
	{
		// Load normally
		if (!CDocument::OnOpenDocument(lpszPathName))
			return FALSE;
	}
	
	return TRUE;
}
